-- written by DaddelZeit
-- DO NOT USE ANY PART OF THIS CODE WITHOUT EXPLICIT PERMISSION

local M = {}

local checktimer = 0
local vehTable = {}
local vehData = {}
local obbTable = {}

local function getCombinedAmbColor(ambcolor)
    local r = ambcolor[1]
    local g = ambcolor[2]
    local b = ambcolor[3]
    return (r+g+b) / 3
end

local function getAmbColor(veh)
    for _, v in pairs(scenetree.findClassObjects("Zone")) do
        local object = scenetree.findObject(v)
        if object then
            if obbTable[object:getId()] and obbTable[object:getId()]:isContained(veh:getPosition()) and object.useAmbientLightColor then
                return getCombinedAmbColor(object.ambientLightColor:toTable())
            elseif not obbTable[object:getId()] then
                local obb = OrientedBox3F()
                obb:set2(object:getTransform(), object:getScale())
                obbTable[object:getId()] = obb
            end
        end
    end

    -- we're sure that the vehicle is not inside a zone at this point
    local sky = scenetree.sunsky
    if not sky and scenetree.findClassObjects("ScatterSky")[1] then scenetree.findObject(scenetree.findClassObjects("ScatterSky")[1]) end
    if sky and sky.ambientScale then return getCombinedAmbColor(sky.ambientScale:toTable()) else return 1 end
end

local function getHighBeam(veh, ambcol)
    -- only do for player because of performance
    if veh:getId() == be:getPlayerVehicle(0):getId() and ambcol < 0.35 then
        if vehData[veh:getId()] and vehData[veh:getId()].lightsState and vehData[veh:getId()].lightsState >= 1 then
            for i = 0, be:getObjectCount()-1 do
                local otherVeh = be:getObject(i)
                if otherVeh:getId() ~= veh:getId() then
                    if vehData[otherVeh:getId()] and (vehData[otherVeh:getId()].lightsState or 0) ~= 0 then
                        local otherVehOBB = otherVeh:getSpawnWorldOOBB()
                        local bbCenter = otherVehOBB:getCenter()
                        local axis0, axis1, axis2 = otherVehOBB:getAxis(0), otherVehOBB:getAxis(1), otherVehOBB:getAxis(2)
                        local halfExtentsX, halfExtentsY, halfExtentsZ = otherVehOBB:getHalfExtents().x, otherVehOBB:getHalfExtents().y, otherVehOBB:getHalfExtents().z

                        local result1,result2 = intersectsRay_OBB(veh:getPosition(), vehData[veh:getId()].dir, bbCenter, (halfExtentsX*15)*axis0, (halfExtentsY*1.4)*axis1, (halfExtentsZ*5)*axis2)
                        if tostring(result1) ~= "inf" and result1 > 0 and result1 < 160 and tostring(result2) ~= "inf" then
                            -- check if the car is behind eg. a wall
                            -- local directionVec = vec3(bbCenter - veh:getPosition())
                            -- local staticRay = castRayStatic(veh:getSpawnWorldOOBB():getCenter(), directionVec, 5000, true)
                            -- if staticRay >= result1 then
                                return 0
                            -- end
                        end
                    end
                end
            end
        end

        return 1
    else
        return 0
    end
end

local function onUpdate(dt)
    checktimer = checktimer + dt
    if checktimer >= 0.5 then
        for _, veh in pairs(vehTable) do
            local vehObj = scenetree.findObjectById(veh)
            if vehObj then
                local ambientColor = getAmbColor(vehObj) or 0
                local lightBaseVal = (ambientColor < 0.7) and 1 or 0
                local highBeamsVal = getHighBeam(vehObj, ambientColor) or 0

                if not vehData[veh] then vehData[veh] = {} end
                if vehData[veh].lastLightBaseVal and vehData[veh].lastHighBeamsVal and (vehData[veh].lastLightBaseVal ~= lightBaseVal or vehData[veh].lastHighBeamsVal ~= highBeamsVal) then
                    vehObj:queueLuaCommand("autoHeadlightsZeit.updateAutoHeadlights("..lightBaseVal + highBeamsVal..")")
                end

                vehData[veh].lastLightBaseVal = lightBaseVal
                vehData[veh].lastHighBeamsVal = highBeamsVal
            end
        end
        checktimer = 0
    end
end

local function updateVehData(vehId, state, dirvecX, dirvecY, dirvecZ)
    if not vehData[vehId] then vehData[vehId] = {} end
    vehData[vehId].lightsState = state
    vehData[vehId].dir = vec3(dirvecX,dirvecY,dirvecZ)
end

local function registerVeh(id)
    local vehDetails = core_vehicles.getVehicleDetails(id)
    if vehDetails and vehDetails.configs and vehDetails.configs.aggregates and vehDetails.configs.aggregates.Years and vehDetails.configs.aggregates.Years.min >= 2013 then
        table.insert(vehTable, id)
    end
end

local function onExtensionLoaded()
    --print("autoLowBeamZeit Loaded")
end

M.updateVehData = updateVehData
M.registerVeh = registerVeh
M.onExtensionLoaded = onExtensionLoaded
M.onUpdate = onUpdate

return M